==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2107_155.zip: ronportal/ron/utils/parser.php <==
<?php
/**
 * stanislav.elbl, 18.05.2016
 * Parser pro nahrazovani promennych v retezci
 */

namespace Ron\Utils;



/**
 * Parser nahrazuje promenne v textu.
 * Promenne maji tvar {$nazev} pripadne {$nazev:format}
 * Mohou to byt pole {$nazev[index]} pripadne {$nazev[index]:format} a index muze obsahovat podvyrazy
 * Reference na promennou se zadavaji takto: {@nazev}
 * Nahrazovani promennych zajistuje objekt, ktery implementuje SymbolTable
 * Preklady pomoci ron\ronText se vkladaji {#text}
 * Class Parser
 * @package Ron\RonPortal\XML
 */
class Parser
{
    const PARSE_STATE_TEXT = 0;
    const PARSE_STATE_LCBR = 1;     // left curly bracket
    const PARSE_STATE_VAR = 2;      // variable (za {$ nebo {@
    const PARSE_STATE_INDEX = 3;    // v promenne za [
    const PARSE_STATE_BEHINDINDEX = 4; // v promenne za [
    const PARSE_STATE_TYPE = 5;     // v promenne za : (datovy typ)
    const PARSE_STATE_END = 10;     // na koncovem symbolu ( podle kontextu }, ] nebo konec retezce)

    /** @var SymbolTable */
    public $symbolTable;


    /**
     * Parser constructor.
     * @param SymbolTable $symbolTable
     */
    public function __construct(SymbolTable $symbolTable)
    {
        $this->symbolTable = $symbolTable;
    }


    /**
     * @param $text
     * @param $pos
     * @param string $formatovani  Muze byt sql, html, nebo prazdne
     * @return mixed
     * @throws \Exception
     */
    private function parsujVariable($text, &$pos, $formatovani = '')
    {
        $state = self::PARSE_STATE_VAR; // varName

        if ($text[$pos] != '$'  &&  $text[$pos] != '@') {
            throw new \Exception('Chyba v metode Parser::parsujVariable!');
        }

        $reference = $text[$pos] == '@'; // priznak, jestli ma vracet referenci
        $pos ++;

        $varName = '';
        $indexstr = '';
        $formatStr = '';

        while ($state != self::PARSE_STATE_END  &&  $pos < strlen($text)) {
            switch ($state) {
                case self::PARSE_STATE_VAR :
                    switch ($text[$pos]) {
                        case '[' :
                            // nacte index ( text az po ] )
                            $pos++;
                            $indexstr = $this->parsujText($text, $pos, '', ']');
                            $pos++; // prejde za ]
                            $state = self::PARSE_STATE_VAR; // POZOR! Neumoznuje vice indexu a pripadny text za indexem se prida k nazvu promenne!
                            break;

                        case ':' :
                            // nacte datovy typ ( text az po } )
                            $pos++;
                            $formatStr = $this->parsujText($text, $pos, '', '}');
                            $state = self::PARSE_STATE_END;
                            break;

                        case '{' :
                            // pravdepodobne zacatek vnorene promenne (jeste musi nasledovat $ nebo @)
                            $pos++;
                            $state = self::PARSE_STATE_LCBR;
                            break;

                        case '}' :
                            $state = self::PARSE_STATE_END;
                            break;

                        default :
                            $varName .= $text[$pos];
                            $pos++;
                            break;
                    }
                    break;

                case self::PARSE_STATE_LCBR : // je za {
                    switch ($text[$pos]) {
                        case '$' :
                        case '@' :
                            $varName .= $this->parsujVariable($text, $pos, ''); // bez formatovani
                            $pos++;
                            break;

                        case '}' :  // nebo ignorovat nebo ukoncit promennou ?
                            $pos++;
                            $varName .= '{}';
                            break;

                        default :
                            $varName .= '{' . $text[$pos];
                            $pos++;
                            break;
                    }
                    $state = self::PARSE_STATE_VAR;
                    break;
            }
        }

        // nacetl obsah promenne az po }
        if ($reference) {
            return $this ->symbolTable ->getVariableReference($varName);
        } else {
            if ($indexstr != '') {
                $value = $this->symbolTable ->getVariableValue($varName);

                if (!isset($value)) {
                    // pokud je zadan format, musi formatovat i null!
                    return $this->symbolTable ->formatValue($value, $formatStr, $formatovani);
                }

                if (is_array($value)) {
                    return $this->symbolTable ->formatValue(isset($value[$indexstr]) ? $value[$indexstr] : null, $formatStr, $formatovani);
                } else if ($value instanceof \ArrayAccess) {
                    // napr. dbrecord
                    return $this->symbolTable ->formatValue($value[$indexstr], $formatStr, $formatovani);
                } else {
                    throw new \Exception('Chyba ve vyrazu: promenna "' . $varName . '" neni pole!');
                }
            }

            $value = $this ->symbolTable ->getVariableValue($varName);
            return $this ->symbolTable ->formatValue($value, $formatStr, $formatovani);
        }

    }


    /**
     * Parsuje cast retezce pro preklad: {#...}
     * @param $text
     * @param $pos
     * @return string
     * @throws \Exception
     */
    private function parsujPreklad($text, &$pos)
    {
        if ($text[$pos] != '#') {
            throw new \Exception('Chyba v metode Parser::parsujPreklad!'); // nemelo by nastat
        }

        $pos ++;

        $result = '';

        while ($pos < strlen($text)  &&  $text[$pos] != '}') {
            $result .= $text[$pos];
            $pos++;
        }

        return \Ron\ronText($result);
    }


    /**
     * Vraci podvyraz ze zadaneho textu zacinajici na $pos. Nahradi vsechny vnitrni podvyrazy {$/@/#...[index]:typ}
     * @param string $text
     * @param int $pos
     * @param string $formatovani   muze byt sql, html, nebo prazdne
     * @param string $stopSymbol    symbol, ktery ukonci parsovani
     * @return string
     * @throws \Exception
     */
    private function parsujText($text, &$pos, $formatovani = '', $stopSymbol = '')
    {
        if (!isset($text)) {
            return '';
        }

        $state = self::PARSE_STATE_TEXT;
        $result = '';

        for (; $pos < strlen($text); $pos++) {
            if ($text[$pos] === $stopSymbol) {
                //$pos++; // presune se za "stopsymbol"
                return $result;
            }

            switch ($state) {
                case self::PARSE_STATE_TEXT :
                    switch($text[$pos]) {
                        case '{' :
                            $state = self::PARSE_STATE_LCBR;
                            break;

                        default :
                            $result .= $text[$pos];
                            break;
                    }

                    break;

                case self::PARSE_STATE_LCBR : // je za {
                    switch ($text[$pos]) {
                        case '$' :
                        case '@' :
                            $result .= $this->parsujVariable($text, $pos, $formatovani);
                            break;

                        case '#' :
                            $result .= $this ->parsujPreklad($text, $pos);
                            break;

                        case '}' :  // nebo ignorovat nebo ukoncit promennou ?
                            $result .= '{}';
                            break;

                        default :
                            $result .= '{' . $text[$pos];
                            break;
                    }
                    $state = self::PARSE_STATE_TEXT;
                    break;

                default :
                    throw new \Exception('Chyba v metode Parser::parsujText! (chybny stav)');
                    break;
            }
        }

        return $result;
    }


    /**
     * @param string $text
     * @param string $formatovani  muze byt sql, html nebo prazdne
     * @return string
     * @throws \Exception
     */
    public function parsuj($text, $formatovani)
    {
        $pos = 0;
        return $this ->parsujText($text, $pos, $formatovani);
    }
}

==> helpdesk.ron.cz?menuid=download&filename=ronportal_4_2206_162.zip: ronportal/ron/utils/parser.php <==
<?php
/**
 * stanislav.elbl, 18.05.2016
 * Parser pro nahrazovani promennych v retezci
 */

namespace Ron\Utils;



/**
 * Parser nahrazuje promenne v textu.
 * Promenne maji tvar {$nazev} pripadne {$nazev:format}
 * Mohou to byt pole {$nazev[index]} pripadne {$nazev[index]:format} a index muze obsahovat podvyrazy
 * Reference na promennou se zadavaji takto: {@nazev}
 * Nahrazovani promennych zajistuje objekt, ktery implementuje SymbolTable
 * Preklady pomoci ron\ronText se vkladaji {#text}
 * Class Parser
 * @package Ron\RonPortal\XML
 */
class Parser
{
    const PARSE_STATE_TEXT = 0;
    const PARSE_STATE_LCBR = 1;     // left curly bracket
    const PARSE_STATE_VAR = 2;      // variable (za {$ nebo {@
    const PARSE_STATE_INDEX = 3;    // v promenne za [
    const PARSE_STATE_BEHINDINDEX = 4; // v promenne za [
    const PARSE_STATE_TYPE = 5;     // v promenne za : (datovy typ)
    const PARSE_STATE_END = 10;     // na koncovem symbolu ( podle kontextu }, ] nebo konec retezce)

    /** @var SymbolTable */
    public $symbolTable;


    /**
     * Parser constructor.
     * @param SymbolTable $symbolTable
     */
    public function __construct(SymbolTable $symbolTable)
    {
        $this->symbolTable = $symbolTable;
    }


    /**
     * @param $text
     * @param $pos
     * @param string $formatovani  Muze byt sql, html, nebo prazdne
     * @return mixed
     * @throws \Exception
     */
    private function parsujVariable($text, &$pos, $formatovani = '')
    {
        $state = self::PARSE_STATE_VAR; // varName

        if ($text[$pos] != '$'  &&  $text[$pos] != '@') {
            throw new \Exception('Chyba v metode Parser::parsujVariable!');
        }

        $reference = $text[$pos] == '@'; // priznak, jestli ma vracet referenci
        $pos ++;

        $varName = '';
        $indexstr = '';
        $formatStr = '';

        while ($state != self::PARSE_STATE_END  &&  $pos < strlen($text)) {
            switch ($state) {
                case self::PARSE_STATE_VAR :
                    switch ($text[$pos]) {
                        case '[' :
                            // nacte index ( text az po ] )
                            $pos++;
                            $indexstr = $this->parsujText($text, $pos, '', ']');
                            $pos++; // prejde za ]
                            $state = self::PARSE_STATE_VAR; // POZOR! Neumoznuje vice indexu a pripadny text za indexem se prida k nazvu promenne!
                            break;

                        case ':' :
                            // nacte datovy typ ( text az po } )
                            $pos++;
                            $formatStr = $this->parsujText($text, $pos, '', '}');
                            $state = self::PARSE_STATE_END;
                            break;

                        case '{' :
                            // pravdepodobne zacatek vnorene promenne (jeste musi nasledovat $ nebo @)
                            $pos++;
                            $state = self::PARSE_STATE_LCBR;
                            break;

                        case '}' :
                            $state = self::PARSE_STATE_END;
                            break;

                        default :
                            $varName .= $text[$pos];
                            $pos++;
                            break;
                    }
                    break;

                case self::PARSE_STATE_LCBR : // je za {
                    switch ($text[$pos]) {
                        case '$' :
                        case '@' :
                            $varName .= $this->parsujVariable($text, $pos, ''); // bez formatovani
                            $pos++;
                            break;

                        case '}' :  // nebo ignorovat nebo ukoncit promennou ?
                            $pos++;
                            $varName .= '{}';
                            break;

                        default :
                            $varName .= '{' . $text[$pos];
                            $pos++;
                            break;
                    }
                    $state = self::PARSE_STATE_VAR;
                    break;
            }
        }

        // nacetl obsah promenne az po }
        if ($reference) {
            return $this ->symbolTable ->getVariableReference($varName);
        } else {
            if ($indexstr != '') {
                $value = $this->symbolTable ->getVariableValue($varName);

                if (!isset($value)) {
                    // pokud je zadan format, musi formatovat i null!
                    return $this->symbolTable ->formatValue($value, $formatStr, $formatovani);
                }

                if (is_array($value)) {
                    return $this->symbolTable ->formatValue(isset($value[$indexstr]) ? $value[$indexstr] : null, $formatStr, $formatovani);
                } else if ($value instanceof \ArrayAccess) {
                    // napr. dbrecord
                    return $this->symbolTable ->formatValue($value[$indexstr], $formatStr, $formatovani);
                } else {
                    throw new \Exception('Chyba ve vyrazu: promenna "' . $varName . '" neni pole!');
                }
            }

            $value = $this ->symbolTable ->getVariableValue($varName);
            return $this ->symbolTable ->formatValue($value, $formatStr, $formatovani);
        }

    }


    /**
     * Parsuje cast retezce pro preklad: {#...}
     * @param $text
     * @param $pos
     * @return string
     * @throws \Exception
     */
    private function parsujPreklad($text, &$pos)
    {
        if ($text[$pos] != '#') {
            throw new \Exception('Chyba v metode Parser::parsujPreklad!'); // nemelo by nastat
        }

        $pos ++;

        // v "identifikatoru" pro preklad mohou byt i promenne
        $result = $this ->parsujText($text, $pos, '', ['}', ':']);

        $originalText = null;
        if ($text[$pos] == ':') {
            // retezec obsahuje i originalni text, ktery se ma pouzit v pripade, ze preklad neexistuje, napr: {$db_stat_popis_CZ:Ceska republika}
            $pos++;
            $originalText = $this ->parsujText($text, $pos, '', '}');
        }

        return \Ron\ronText($result, $originalText);
    }


    /**
     * Vraci podvyraz ze zadaneho textu zacinajici na $pos. Nahradi vsechny vnitrni podvyrazy {$/@/#...[index]:typ}
     * @param string $text
     * @param int $pos
     * @param string $formatovani   muze byt sql, html, nebo prazdne
     * @param string $stopSymbol    symbol, ktery ukonci parsovani
     * @return string
     * @throws \Exception
     */
    private function parsujText($text, &$pos, $formatovani = '', $stopSymbol = '')
    {
        if (!isset($text)) {
            return '';
        }

        $state = self::PARSE_STATE_TEXT;
        $result = '';

        for (; $pos < strlen($text); $pos++) {
            if (is_array($stopSymbol)  &&  in_array($text[$pos], $stopSymbol, true)) {
                return $result;
            } else if ($text[$pos] === $stopSymbol) {
                return $result;
            }

            switch ($state) {
                case self::PARSE_STATE_TEXT :
                    switch($text[$pos]) {
                        case '{' :
                            $state = self::PARSE_STATE_LCBR;
                            break;

                        default :
                            $result .= $text[$pos];
                            break;
                    }

                    break;

                case self::PARSE_STATE_LCBR : // je za {
                    switch ($text[$pos]) {
                        case '$' :
                        case '@' :
                            $result .= $this->parsujVariable($text, $pos, $formatovani);
                            break;

                        case '#' :
                            $result .= $this ->parsujPreklad($text, $pos);
                            break;

                        case '}' :  // nebo ignorovat nebo ukoncit promennou ?
                            $result .= '{}';
                            break;

                        default :
                            $result .= '{' . $text[$pos];
                            break;
                    }
                    $state = self::PARSE_STATE_TEXT;
                    break;

                default :
                    throw new \Exception('Chyba v metode Parser::parsujText! (chybny stav)');
                    break;
            }
        }

        return $result;
    }


    /**
     * @param string $text
     * @param string $formatovani  muze byt sql, html nebo prazdne
     * @return string
     * @throws \Exception
     */
    public function parsuj($text, $formatovani)
    {
        $pos = 0;
        return $this ->parsujText($text, $pos, $formatovani);
    }
}
